#include "XWindowGraphicsContext.h"
#include "DefaultDisplay.h"

#include <ThreadPrimitives.h>

#include <boost/bind.hpp>

XExtendedGLFuncs::XExtendedGLFuncs()
{
	glBlendColor = (ExtendedGLFuncs::glBlendColorFunc)glXGetProcAddress((const GLubyte*)"glBlendColor");
}

XExtendedGLFuncs::~XExtendedGLFuncs()
{
}

static XExtendedGLFuncs s_extGLFuncs;

void XWindowGraphicsContext::SwapBuffers()
{
	glXSwapBuffers(m_disp, m_glxWin);
}

XWindowGraphicsContext::XWindowGraphicsContext(Window win, const GLXFBConfig& config, int width, int height)
: m_config(config)
{
	Mutex::ScopedLock lock(m_monitorLock);

	m_disp = GetDefaultDisplay();
	ASSERT(m_disp != NULL);

	m_glxWin = glXCreateWindow(m_disp, config, win, NULL);

	m_context = glXCreateNewContext(m_disp, m_config, GLX_RGBA_TYPE, NULL, True);
	ASSERT(m_context != NULL);

	m_glServer = new OpenGLCommandServer(width, height, boost::bind(&XWindowGraphicsContext::SwapBuffers, this), s_extGLFuncs);
}

XWindowGraphicsContext::~XWindowGraphicsContext()
{
	delete m_glServer;

	glXDestroyContext(m_disp, m_context);
	glXDestroyWindow(m_disp, m_glxWin);
}

void XWindowGraphicsContext::SetCurrentContext()
{
	glXMakeCurrent(m_disp, m_glxWin, m_context);
}
