#ifdef _WIN32_WINNT
#undef _WIN32_WINNT
#define _WIN32_WINNT 0x0400
#else
#define _WIN32_WINNT 0x0400
#endif

#include <windows.h>

#include "ThreadPrimitives.h"


static DWORD WINAPI StartWinThreadFunc(LPVOID param)
{
	ThreadPlat* thread = static_cast<ThreadPlat*>(param);

	thread->m_func();

	return 0;
}

ThreadPlat::ThreadPlat(const boost::function<void ()>& func)
: m_func(func)
{
}

Thread::Thread(const ThreadFunc& func)
	: m_threadPlat(func)
{
	m_threadPlat.m_thread = CreateThread(NULL, 0, StartWinThreadFunc, &m_threadPlat, 0, &m_threadPlat.m_threadId);
	ASSERT(m_threadPlat.m_thread != NULL);
};

Thread::~Thread()
{
	DWORD result = WaitForMultipleObjects(1, &m_threadPlat.m_thread, FALSE, INFINITE);
	ASSERT(result == WAIT_OBJECT_0);
};

Thread::ID Thread::GetID() const
{
	return m_threadPlat.m_threadId;
}


Thread::ID Thread::GetCurrentID()
{
	return GetCurrentThreadId();
}

	
Semaphore::Semaphore(int initialCount)
{
	m_semPlat.m_sem = CreateSemaphore(NULL, initialCount, 99999, NULL);
	ASSERT(m_semPlat.m_sem != 0);
}
Semaphore::~Semaphore()
{
	BOOL result = CloseHandle(m_semPlat.m_sem);
	ASSERT(result == TRUE);
}

void Semaphore::P()
{
	DWORD result = WaitForSingleObject(m_semPlat.m_sem, INFINITE);
	ASSERT(result == WAIT_OBJECT_0);
}
void Semaphore::V()
{
	BOOL result = ReleaseSemaphore(m_semPlat.m_sem, 1, NULL);
	ASSERT(result == TRUE);
}

Mutex::Mutex()
{
	m_mutexPlat.m_locked = false;

	m_mutexPlat.m_sem = CreateSemaphore(NULL, 1, 99999, NULL);
	ASSERT(m_mutexPlat.m_sem != 0);
}
Mutex::~Mutex()
{
	BOOL result = CloseHandle(m_mutexPlat.m_sem);
	ASSERT(result == TRUE);
}

void Mutex::Lock()
{
	DWORD result = WaitForSingleObject(m_mutexPlat.m_sem, INFINITE);
	ASSERT(result == WAIT_OBJECT_0);

	ASSERT(!m_mutexPlat.m_locked);
	m_mutexPlat.m_locked = true;
}

void Mutex::Unlock()
{
	ASSERT(m_mutexPlat.m_locked);
	m_mutexPlat.m_locked = false;

	BOOL result = ReleaseSemaphore(m_mutexPlat.m_sem, 1, NULL);
	ASSERT(result == TRUE);
}

bool Mutex::IsLocked() const
{
	return m_mutexPlat.m_locked;
}


Condition::Condition()
{
	m_condPlat.m_waitingThreads = 0;

	m_condPlat.m_condLock = CreateSemaphore(NULL, 1, 99999, NULL);
	ASSERT(m_condPlat.m_condLock != 0);
	m_condPlat.m_waitSem = CreateSemaphore(NULL, 0, 99999, NULL);
	ASSERT(m_condPlat.m_condLock != 0);
}
Condition::~Condition()
{
	BOOL result = CloseHandle(m_condPlat.m_condLock);
	ASSERT(result == TRUE);

	result = CloseHandle(m_condPlat.m_waitSem);
	ASSERT(result == TRUE);
}

void Condition::Wait(Mutex& mutex)
{
	DWORD result = WaitForSingleObject(m_condPlat.m_condLock, INFINITE);
	ASSERT(result == WAIT_OBJECT_0);

	++m_condPlat.m_waitingThreads;

	mutex.Unlock();

	result = SignalObjectAndWait(m_condPlat.m_condLock, m_condPlat.m_waitSem, INFINITE, FALSE);
	ASSERT(result == WAIT_OBJECT_0);

	mutex.Lock();
}

void Condition::SignalOne()
{
	DWORD result = WaitForSingleObject(m_condPlat.m_condLock, INFINITE);
	ASSERT(result == WAIT_OBJECT_0);

	if(m_condPlat.m_waitingThreads > 0)
	{
		BOOL ret = ReleaseSemaphore(m_condPlat.m_waitSem, 1, NULL);
		ASSERT(ret == TRUE);

		--m_condPlat.m_waitingThreads;
	}

	BOOL ret = ReleaseSemaphore(m_condPlat.m_condLock, 1, NULL);
	ASSERT(ret == TRUE);
}

void Condition::SignalAll()
{
	DWORD result = WaitForSingleObject(m_condPlat.m_condLock, INFINITE);
	ASSERT(result == WAIT_OBJECT_0);

	if(m_condPlat.m_waitingThreads > 0)
	{
		BOOL ret = ReleaseSemaphore(m_condPlat.m_waitSem, m_condPlat.m_waitingThreads, NULL);
		ASSERT(ret == TRUE);

		m_condPlat.m_waitingThreads = 0;
	}

	BOOL ret = ReleaseSemaphore(m_condPlat.m_condLock, 1, NULL);
	ASSERT(ret == TRUE);
}