#ifndef KEYBOARDINPUT_H_
#define KEYBOARDINPUT_H_

#include <ThreadPrimitives.h>

#include <PotentialObject.h>
#include <Events.h>
#include <boost/function.hpp>

class KeyboardInput
{
public:
	enum KeyType
	{
		KT_INVALID=-1,

		KT_ESC,
		KT_F1,
		KT_F2,
		KT_F3,
		KT_F4,
		KT_F5,
		KT_F6,
		KT_F7,
		KT_F8,
		KT_F9,
		KT_F10,
		KT_F11,
		KT_F12,
		KT_PAUSE,
		KT_TILDE,
		KT_1,
		KT_2,
		KT_3,
		KT_4,
		KT_5,
		KT_6,
		KT_7,
		KT_8,
		KT_9,
		KT_0,
		KT_MINUS,
		KT_EQUAL,
		KT_BACKSPACE,
		KT_TAB,
		KT_Q,
		KT_W,
		KT_E,
		KT_R,
		KT_T,
		KT_Y,
		KT_U,
		KT_I,
		KT_O,
		KT_P,
		KT_SQUARE_BRACKET_L,
		KT_SQUARE_BRACKET_R,
		KT_BACKSLASH,
		KT_CAPSLOCK,
		KT_A,
		KT_S,
		KT_D,
		KT_F,
		KT_G,
		KT_H,
		KT_J,
		KT_K,
		KT_L,
		KT_SEMICOLON,
		KT_APOSTROPHE,
		KT_ENTER,
		KT_SHIFT,
		KT_Z,
		KT_X,
		KT_C,
		KT_V,
		KT_B,
		KT_N,
		KT_M,
		KT_COMMA,
		KT_PERIOD,
		KT_SLASH,
		KT_CTRL,
		KT_SPACE,
		KT_LEFT,
		KT_RIGHT,
		KT_UP,
		KT_DOWN,
		KT_INSERT,
		KT_DELETE,
		KT_HOME,
		KT_END,
		KT_PG_UP,
		KT_PG_DOWN,
	};

	struct KeyEvent
	{
		KeyType key;
		bool pressed;
	};
	typedef void KeyEventSig(const KeyEvent&);

	class KeyListenerConnection
	{
		KeyboardInput& m_input;
		PotentialObject<Event<KeyEventSig>::Connection> m_conn;

	public:
		KeyListenerConnection(KeyboardInput& input, const boost::function<KeyEventSig>& handler)
			: m_input(input)
		{
			Mutex::ScopedLock lock(m_input.m_handlerLock);

			new(m_conn) Event<KeyEventSig>::Connection(m_input.m_event, handler);
		}
		~KeyListenerConnection()
		{
			Mutex::ScopedLock lock(m_input.m_handlerLock);
			m_conn.Clear();
		}
	};
	friend class KeyListenerConnection;

private:
	Mutex m_handlerLock;
	Event<KeyEventSig> m_event;

protected:
	void PostKeyPress(const KeyEvent& keyPress);

public:
	KeyboardInput()
	{};
	virtual ~KeyboardInput()
	{};
};

#endif
