#ifndef THREEDEEMODEL_H_
#define THREEDEEMODEL_H_

#include <Materials.h>
#include <TriangleMesh.h>

#include <Algebra.h>
#include <ResourceManager.h>

#include <vector>

#include "OBJFile.h"

class GraphicsFrame;
class Model
{
	struct MaterialMesh
	{
		Material* material;
		TriangleMesh mesh;
	};
	typedef std::vector<MaterialMesh> MaterialMeshList;
	MaterialMeshList m_materialMeshList;

	typedef std::map<Material*, Material*> MaterialMaterialMap;
	MaterialMaterialMap m_toWireframe; // Maps normal material to wireframe version

	SafePointerList<Material> m_materials;

public:
	Model(const OBJFile::Object& objFileObject); // Create a model from an OBJ file

	void AddToGraphicsFrame(GraphicsFrame& frame, const Matrix<4,4>& modelView, float alpha=1.0f, int layer=0, bool wireframe=false) const;
};

// Use this resource factory to obtain a model object from an OBJ file
class ModelFromObjFactory : TypedResourceFactory<Model>
{
	ResourceManager& m_resMan;
public:
	ModelFromObjFactory(ResourceManager& resMan)
		: m_resMan(resMan)
	{};
	virtual void CreateObject(void* mem, std::istream& in)
	{
		new(mem) Model(OBJFile(in, m_resMan).GetObjectItem(0));
	}
};

#endif
