#ifndef MOUSEINPUT_H_
#define MOUSEINPUT_H_

#include "ThreadPrimitives.h"

#include <boost/function.hpp>

class MouseInput
{
public:

	struct MoveEvent
	{
		int dx;
		int dy;
	};
	typedef boost::function<void (const MoveEvent&)> MoveEventHandler;

	struct ButtonEvent
	{
		int button;
		bool pressed;
	};
	typedef boost::function<void (const ButtonEvent&)> ButtonEventHandler;
	
	class MoveListenerConnection
	{
		MouseInput& m_input;

	public:
		MoveListenerConnection(MouseInput& input, const MoveEventHandler& handler)
			: m_input(input)
		{
			m_input.RegisterMoveHandler(handler);
		}
		~MoveListenerConnection()
		{
			m_input.DeRegisterMoveHandler();
		}
	};

	class ButtonListenerConnection
	{
		MouseInput& m_input;

	public:
		ButtonListenerConnection(MouseInput& input, const ButtonEventHandler& handler)
			: m_input(input)
		{
			m_input.RegisterButtonHandler(handler);
		}
		~ButtonListenerConnection()
		{
			m_input.DeRegisterButtonHandler();
		}
	};

private:
	Mutex m_handlerLock;

	MoveEventHandler m_velHandler;
	ButtonEventHandler m_buttonHandler;

	void RegisterMoveHandler(const MoveEventHandler& handler);
	void DeRegisterMoveHandler();
	void RegisterButtonHandler(const ButtonEventHandler& handler);
	void DeRegisterButtonHandler();

protected:
	void PostMoveEvent(const MoveEvent& velChange);
	void PostButtonEvent(const ButtonEvent& button);

public:
	MouseInput()
		: m_velHandler(0)
		, m_buttonHandler(0)
	{};

	virtual ~MouseInput()
	{};

};

#endif
