#ifndef OPENGLCOMMANDS_H_
#define OPENGLCOMMANDS_H_

#include <GraphicsFrame.h>
#include <Algebra.h>
#include <TriangleMesh.h>
#include <set>

// We define some functionality here that will be common to any GraphicsContext that
// uses OpenGL
class OpenGLRenderContext
{

	void BeginFrame(const Vector<3>& fillColour, const Matrix<4,4>& projMatrix);
	void EndFrame();

	void ProcessPacket(const GraphicsFrame::Packet& packet);

	bool DisplayListExists(const TriangleMesh* mesh, bool wireframe) const;
	void CreateDisplayList(const TriangleMesh* mesh, bool wireframe);
	int GetDisplayList(const TriangleMesh* mesh, bool wireframe) const;

	void ApplyAlpha(float alpha);
	void EnableAlpha();
	void DisableAlpha();

	void ResetLayerInfo();

	void LoadMaterial(const Material* mat, bool forceReload=false);

	const Material* m_curMaterial; // What the current context has set as the material
	

	typedef std::map<const TriangleMesh*, int> DispListMap;
	DispListMap m_dispListMap; // Maps meshes to their display lists
	DispListMap m_dispListMapWireframe; // Maps meshes to their wireframe display lists

	float m_prevAlpha;

	int m_curLayer;

	bool m_lightsEnabled;

public:
	OpenGLRenderContext();

	struct RenderOptions
	{
		bool usePostMultMatrix;
		Matrix<4,4> postMultMatrix;

		RenderOptions()
			: usePostMultMatrix(false)
		{};
	};

	void SetViewportSize(int width, int height);
	void RenderGraphicsFrame(GraphicsFrame& frame, const Matrix<4,4>& projMatrix, const RenderOptions& options = RenderOptions());
};

#endif
