#ifndef SCOPEDFSTREAM_H_
#define SCOPEDFSTREAM_H_

#include <Exceptions.h>

#include <fstream>
#include <sstream>

template<class STREAM>
struct ScopedFStream
{
	class IOException : public std::exception
	{
		std::string m_msg;

	public:
		IOException(const std::string& msg)
			: m_msg(msg)
		{};

		const char* what() const {return m_msg.c_str();}
	};

	STREAM s;

	ScopedFStream(const std::string& fname)
	{
		s.open(fname.c_str());
		if(s.fail())
		{
			std::ostringstream oss;
			oss << "Error opening file " << fname << ".\n";
			throw IOException(oss.str());
		}
	}

	~ScopedFStream()
	{
		s.close();
	}
};

#endif
