#ifndef THREADPRIMITIVES_H_
#define THREADPRIMITIVES_H_

#include "ThreadPlat.h" // Include some platform-specific definitions

#include <boost/function.hpp>


class Thread
{
public:
	typedef boost::function<void ()> ThreadFunc;
	typedef unsigned int ID;

private:
	ThreadPlat m_threadPlat;

public:	
	Thread(const ThreadFunc& func);
	~Thread();

	ID GetID() const; // Returns the ID of this thread

	static ID GetCurrentID(); // Returns the ID of the calling Thread
};

class Semaphore
{
	SemaphorePlat m_semPlat;

public:
	Semaphore(int initialCount);
	~Semaphore();

	void P();
	void V();
};

class Mutex
{
	MutexPlat m_mutexPlat;

	public:
	Mutex();
	~Mutex();

	void Lock();
	void Unlock();

	bool IsLocked() const;
	
	class ScopedLock
	{
		Mutex& m_mutex;

		public:
		ScopedLock(Mutex& mutex)
		: m_mutex(mutex)
		{
			m_mutex.Lock();
		}
		~ScopedLock()
		{
			m_mutex.Unlock();
		}

		Mutex& GetMutex() {return m_mutex;}
	};
};

class Condition
{
	ConditionPlat m_condPlat;

public:
	Condition();
	~Condition();

	void Wait(Mutex& mutex);
	void SignalOne();
	void SignalAll();
};

#endif
