#include "FileFetcherLoadWorker.h"
#include <BulkFileIO.h>

#include <boost/bind.hpp>

FileFetcherLoadWorker::FileFetcherLoadWorker(FileFetcher& fetcher)
: m_fetcher(fetcher)
{
	m_workThread = new Thread(boost::bind(&FileFetcherLoadWorker::Run, this));
}

FileFetcherLoadWorker::~FileFetcherLoadWorker()
{
	delete m_workThread;
}


void FileFetcherLoadWorker::Run()
{
	while(true)
	{
		FileFetcher::WorkUnit* work = m_fetcher.RequestWork();

		if(work == 0) break; // The quit signal

		DoFileLoad(work);

		m_fetcher.ReturnWorkResults(work);
	}
}

void FileFetcherLoadWorker::DoFileLoad(FileFetcher::WorkUnit* work)
{
	BulkFileIO::LoadResults loadResults = BulkFileIO::LoadFileToMemory(work->resId, m_fetcher.GetFileAllocator());

	work->dataSize = loadResults.dataSize;
	work->data = loadResults.data;
}
