#include "Projection.h"

#include <R3Utils.h>

Matrix<4,4> MakePerspectiveProjectionMatrix(float fov, float aspectRatio, float n, float f)
{
	float fovRad = 2*M_PI * fov / 360.0f; 

	float t = n * tan(fovRad/2.0f);
	float b = -t;
	float r = t * aspectRatio;
	float l = -r;

	Matrix<4,4> proj(MakeZeroMatrix());
	proj.GetVal(0,0) = 2*n / (r - l);
	proj.GetVal(1,1) = 2*n / (t - b);
	proj.GetVal(0,2) = (r+l) / (r-l);
	proj.GetVal(1,2) = (t+b) / (t-b);
	proj.GetVal(2,2) = - (f + n) / (f - n);
	proj.GetVal(3,2) = -1;
	proj.GetVal(2,3) = - (2 * f * n) / (f - n);

	return proj;
}

Matrix<4,4> MakeOrthographicProjectionMatrix(float left, float right, float bottom, float top, float near, float far)
{
	float widthRecip = 1 / (right - left);
	float heightRecip = 1 / (top - bottom);
	float depthRecip = -1 / (far - near);

	Matrix<4,4> proj(Matrix<4,4>::Identity);
	proj.GetVal(0,0) = 2 * widthRecip;
	proj.GetVal(1,1) = 2 * heightRecip;
	proj.GetVal(2,2) = 2 * depthRecip;
	proj.GetVal(3,3) = 1.0f;

	proj.GetVal(0,3) = (right + left) * widthRecip;
	proj.GetVal(1,3) = (top + bottom) * heightRecip;
	proj.GetVal(2,3) = (far + near) * depthRecip;

	return proj;
}
