#include "ResourceCreatorWorker.h"
#include <FileFetcher.h>

#include <iostream>
#include <sstream>

ResourceCreatorWorker::ResourceCreatorWorker(ResourceCreator& creator, int workerId, ThreadSafeAllocator& allocator)
	: m_creator(creator)
	, m_workerId(workerId)
	, m_allocator(allocator)
{
	m_thread = new Thread(boost::bind(&ResourceCreatorWorker::Run, this));
}

ResourceCreatorWorker::~ResourceCreatorWorker()
{
	delete m_thread;
}


void ResourceCreatorWorker::Run()
{
	while(true)
	{
		ResourceCreator::WorkOrder* workOrder = m_creator.WorkerGetWork(m_workerId);

		if(workOrder == 0) break;

		ProcessWorkOrder(workOrder);

		m_creator.WorkerDoneWork(workOrder, m_workerId);
	}
}

void ResourceCreatorWorker::ProcessWorkOrder(ResourceCreator::WorkOrder* workOrder)
{
	std::istringstream iss((char*)workOrder->file->GetData(), std::ios::binary);

	workOrder->processedData = m_allocator.Allocate(workOrder->resAlloc.ObjectSize());

	try
	{
		workOrder->resAlloc.CreateObject(workOrder->processedData, iss);
	}
	catch(std::exception& e)
	{
		std::cerr << "Exception throw creating object '" << workOrder->creationProcess->GetResourceID().GetFilepath().string() << "':" << std::endl;
		std::cerr << e.what() << std::endl;

		m_allocator.Free(workOrder->processedData);
		workOrder->processedData = 0;
	}
}
