#include <CommandQueue.h>
#include <iostream>

DECL_COMMAND(PrintCommand)
{
	COMMAND_BODY(PrintCommand)

	std::string msg;

	PrintCommand(const std::string& pMsg)
		: msg(pMsg)
	{};
};
DECL_COMMAND(AddAndPrintCommand)
{
	COMMAND_BODY(AddAndPrintCommand)

	int a, b;

	AddAndPrintCommand(int pA, int pB)
		: a(pA), b(pB)
	{};
};

DECL_COMMAND(ResetAccumulatorCommand)
{
	COMMAND_BODY(ResetAccumulatorCommand)
};
DECL_COMMAND(AddAccumulatorCommand)
{
	COMMAND_BODY(AddAccumulatorCommand)

	int num;
	AddAccumulatorCommand(int pNum) : num(pNum) {};
};
DECL_COMMAND(PrintAccumulatorCommand)
{
	COMMAND_BODY(PrintAccumulatorCommand)
};

class TestCommandHandler : HANDLES(PrintCommand), HANDLES(AddAndPrintCommand), 
							HANDLES(ResetAccumulatorCommand), HANDLES(AddAccumulatorCommand),
							HANDLES(PrintAccumulatorCommand)
{
	Mutex m_monitorLock;

	CommandQueue m_commandQueue;


	void ProcessCommand(PrintCommand& cmd)
	{
		Mutex::ScopedLock lock(m_monitorLock);
		std::cout << "PRINT: " << cmd.msg << std::endl;
	}
	void ProcessCommand(AddAndPrintCommand& cmd)
	{
		Mutex::ScopedLock lock(m_monitorLock);
		std::cout << "ADD AND PRINT: " << cmd.a << " + " << cmd.b << " = " << cmd.a + cmd.b << std::endl;
	}


	int m_accumulator;
	void ProcessCommand(ResetAccumulatorCommand& cmd)
	{
		Mutex::ScopedLock lock(m_monitorLock);
		m_accumulator = 0;
	}
	void ProcessCommand(AddAccumulatorCommand& cmd)
	{
		Mutex::ScopedLock lock(m_monitorLock);
		m_accumulator += cmd.num;
	}
	void ProcessCommand(PrintAccumulatorCommand& cmd)
	{
		Mutex::ScopedLock lock(m_monitorLock);
		std::cout << "ACCUMULATOR: " << m_accumulator << std::endl;
	}

public:

	TestCommandHandler(int maxCommands, int maxMem)
		: m_commandQueue(maxCommands, maxMem)
	{
	}

	CommandQueue& GetCommandQueue() {return m_commandQueue;}
};

static void AccumTest(TestCommandHandler& ch)
{
	ADDCOMMAND2(ch, ResetAccumulatorCommand());
	for(int i=0; i<20; ++i)
	{
		ADDCOMMAND2(ch, AddAccumulatorCommand(5));
	}
	ADDCOMMAND2(ch, PrintAccumulatorCommand());

	ADDCOMMAND2(ch, ResetAccumulatorCommand());
	for(int i=0; i<40; ++i)
	{
		for(int j=0; j<100; ++j)
		{
			ADDCOMMAND2(ch, AddAccumulatorCommand(1));
		}
		ADDCOMMAND2(ch, PrintAccumulatorCommand());
	}
}

int main(int argc, char** args)
{
	{
		std::cout << "Constructing command queue 1" << std::endl;

		TestCommandHandler testHandler(2, 100);

		ADDCOMMAND2(testHandler, PrintCommand("Test1"));
		ADDCOMMAND2(testHandler, PrintCommand("Test2"));
	}

	std::cout << "Done test 1" << std::endl;

	{
		std::cout << "Constructing command queue 2" << std::endl;

		TestCommandHandler testHandler(2,100);

		ADDCOMMAND2(testHandler, PrintCommand("Test1"));
		ADDCOMMAND2(testHandler, PrintCommand("Test2"));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(5,9));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(1,1));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(1,1));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(2,2));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(3,3));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(5,5));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(8,8));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(13,13));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(21,21));
		ADDCOMMAND2(testHandler, AddAndPrintCommand(34,34));

		std::cout << "Does this print statement garble or come out first?  It might." << std::endl;
	}
	std::cout << "But regardless, this print statement is last." << std::endl;

	std::cout << "Done test 2" << std::endl;


	{
		TestCommandHandler testHandler(1,100);
		AccumTest(testHandler);
		ADDCOMMAND2(testHandler, PrintCommand("Done."));
	}

	std::cout << "Done test 3" << std::endl;

	{
		TestCommandHandler testHandler(100,30);
		AccumTest(testHandler);
	}

	std::cout << "Done test 4" << std::endl;
	{
		TestCommandHandler testHandler(1000,20000);
		AccumTest(testHandler);
		ADDCOMMAND2(testHandler, PrintCommand("Done."));
	}

	std::cout << "Done test 5" << std::endl;

	{
		TestCommandHandler testHandler(1,30);
		AccumTest(testHandler);
	}

	std::cout << "Done test 6" << std::endl;

	return 0;
}

