#include <OSWindow.h>
#include <KeyboardInput.h>
#include <MouseInput.h>
#include <GraphicsContext.h>

#include <System.h>
#include <R3Utils.h>

#include <GL/gl.h>

#include <iostream>

void RenderPyramid(const Matrix<4,4>& modelView)
{
	const Vector<3> p1(0,0,0);
	const Vector<3> p2(1,0,0);
	const Vector<3> p3(1,0,1);
	const Vector<3> p4(0,0,1);
	const Vector<3> p5(0.5,1,0.5);

	const Vector<3> n1( Cross(p5-p1, p2-p1) );
	const Vector<3> n2( Cross(p5-p2, p3-p2) );
	const Vector<3> n3( Cross(p5-p3, p4-p3) );
	const Vector<3> n4( Cross(p5-p4, p1-p4) );
	const Vector<3> n5( Cross(p1-p4, p3-p4) );
	const Vector<3> n6( Cross(p1-p3, p2-p3) );

	glLoadMatrixf(modelView.GetElements());

	glBegin(GL_TRIANGLES);

	glNormal3fv(n1.GetElements());
	glVertex3fv(p5.GetElements());
	glVertex3fv(p2.GetElements());
	glVertex3fv(p1.GetElements());

	glNormal3fv(n2.GetElements());
	glVertex3fv(p5.GetElements());
	glVertex3fv(p3.GetElements());
	glVertex3fv(p2.GetElements());

	glNormal3fv(n3.GetElements());
	glVertex3fv(p5.GetElements());
	glVertex3fv(p4.GetElements());
	glVertex3fv(p3.GetElements());


	glNormal3fv(n4.GetElements());
	glVertex3fv(p5.GetElements());
	glVertex3fv(p1.GetElements());
	glVertex3fv(p4.GetElements());

	glNormal3fv(n5.GetElements());
	glVertex3fv(p1.GetElements());
	glVertex3fv(p3.GetElements());
	glVertex3fv(p4.GetElements());

	glNormal3fv(n6.GetElements());
	glVertex3fv(p1.GetElements());
	glVertex3fv(p2.GetElements());
	glVertex3fv(p3.GetElements());

	glEnd();
}

static bool s_quitFlag = false;
static float s_speed = 1.0f;
static bool s_toggleFullscreen = false;

static void ProcessInput(const KeyboardInput::KeyEvent& key)
{
	if(!key.pressed) return;

	switch(key.key)
	{
	case KeyboardInput::KT_UP:
		s_speed += 0.1f;
		break;

	case KeyboardInput::KT_DOWN:
		s_speed -= 0.1f;
		break;

	case KeyboardInput::KT_F:
		// toggle fullscreen
		s_toggleFullscreen = true;
		break;

	case KeyboardInput::KT_ESC:
		s_quitFlag = true;
		break;

	default: break;
	};
}

static void ProcessMouseMove(const MouseInput::MoveEvent& velEvent)
{
	std::cout << "velEvent: (" << velEvent.x << ", " << velEvent.y << ")" << std::endl;
}

static void ProcessMouseButton(const MouseInput::ButtonEvent& buttonEvent)
{
	std::cout << "buttonEvent: (" << buttonEvent.button << ", " << buttonEvent.pressed << ")" << std::endl;
}

int main(int argc, char** args)
{
	OSWindow win1("Test window 1", 640,480);

//	OSWindow win2("Test window 2", 800,600);

	KeyboardInput::KeyListenerConnection keyConn1(win1.GetKeyboardInput(), &ProcessInput);
//	KeyboardInput::KeyListenerConnection keyConn2(win2.GetKeyboardInput(), &ProcessInput);

	MouseInput::MoveListenerConnection mouseMoveConn1(win1.GetMouseInput(), &ProcessMouseMove);
//	MouseInput::MoveListenerConnection mouseMoveConn2(win2.GetMouseInput(), &ProcessMouseMove);

	MouseInput::ButtonListenerConnection mouseButtonConn1(win1.GetMouseInput(), &ProcessMouseButton);
//	MouseInput::ButtonListenerConnection mouseButtonConn2(win2.GetMouseInput(), &ProcessMouseButton);

	float lastFPSUpdate = 0;
	int frames = 0;

	float curTime = System::GetTimeInSeconds();
//	while((!win1.WantsToQuit() || !win2.WantsToQuit()) && !s_quitFlag)
	while((!win1.WantsToQuit()) && !s_quitFlag)
	{
		float nextTime = System::GetTimeInSeconds();
		float timeDiff = nextTime - curTime;
		curTime = nextTime;

		lastFPSUpdate += timeDiff;
		if(lastFPSUpdate > 1)
		{
//			std::cout << "FPS: " << frames / lastFPSUpdate << std::endl;
			frames = 0;
			lastFPSUpdate = 0;
		}
		++frames;

		static float angle1 = 0;
		static float angle2 = 0;
		angle1 += timeDiff * s_speed * 2.6f;
		angle2 += -timeDiff * s_speed * 2.0f;

		if(s_toggleFullscreen && !win1.WantsToQuit())
		{
			s_toggleFullscreen = false;

			win1.SetWindowMode((win1.GetWindowMode() == OSWindow::M_WINDOWED ? OSWindow::M_FULLSCREEN : OSWindow::M_WINDOWED));
		}
		win1.GetGraphicsContext().FlushWait();

		win1.GetGraphicsContext().StartFrame();
		win1.GetGraphicsContext().SubmitRenderCall(RenderPyramid, MakeTranslationMatrix(0,0,-4.0f)*MakeRotationMatrix(angle1, 'z'));
		win1.GetGraphicsContext().EndFrame();

/*
		win2.GetGraphicsContext().FlushWait();

		win2.GetGraphicsContext().StartFrame();
		win2.GetGraphicsContext().SubmitRenderCall(RenderPyramid, MakeRotationMatrix(angle2, 'z'));
		win2.GetGraphicsContext().EndFrame();
*/
	}

	return 0;
}
