#ifndef EDITCAMERA_H_
#define EDITCAMERA_H_

#include <Algebra.h>

//
// This camera is used by the editor windows for editing objects.  The camera
// is always focused on a specific point in world space.  This point can be moved
// around, and by doing so, the camera moves as well.  The camera is always looking
// at the focus point from a certain angle, a certain distance away.  These three
// variables are all maintained inside of the EditCamera.
//

class EditCamera
{
	UnitQuaternion m_rotation; // Our current rotation around the focus point

	Vector<4> m_focusPoint; // The point in world space that the camera is looking at

	float m_distance; // The distance away from the focal point the camera is

public:
	EditCamera();

	const Vector<4>& GetFocusPoint() const {return m_focusPoint;}
	void SetFocusPoint(const Vector<4>& focusPoint) {m_focusPoint = focusPoint;}

	float GetDistance() const {return m_distance;}
	void SetDistance(float distance) {m_distance = distance;}

	const UnitQuaternion& GetRotation() const {return m_rotation;}
	void SetRotation(const UnitQuaternion& rotation) {m_rotation = rotation;}

	Matrix<4,4> GetModelView() const;
};

#endif
