#ifndef MAINSYSTEMFRAME_H_
#define MAINSYSTEMFRAME_H_

#include "ToolWorldDefinition.h"
#include <wx/wx.h>

#include <memory>

class ResourceManager;
class WorldDefinition;
class SymbolViewPanel;

class MainSystemFrame : public wxFrame
{
	enum
	{
		ADD_SYMBOL_BUTTON = 100,
		REM_SYMBOL_BUTTON,
		SYMBOL_LISTBOX,
		ID_NEW,
		ID_SAVEAS,
		ID_SAVE,
		ID_OPEN,
		ID_QUIT,
		ID_PREVIEW,
	};

	void OnAddSymbol(wxCommandEvent&);
	void OnRemoveSymbol(wxCommandEvent&);
	void OnSymbolSelected(wxCommandEvent&);

	PotentialObject<ToolWorldDefinition> m_world;

	PotentialObject<Event<ToolWorldDefinition::SymbolAddedSig>::Connection> m_symAddedConn;
	void HandleSymbolAdded(const ToolSymbol* sym);

	PotentialObject<Event<ToolWorldDefinition::SymbolRemovedSig>::Connection> m_symRemovedConn;
	void HandleSymbolRemoved(const ToolSymbol* sym);

	wxListBox* m_symbolList;

	wxPanel* m_selectedSymbolPlaceholder;
	wxBoxSizer* m_symbolPlaceholderSizer;

	SymbolViewPanel* m_symbolView;
	
	ResourceManager& m_resMan;

	void OnNew(wxCommandEvent&);
	void OnSaveAs(wxCommandEvent&);
	void OnSave(wxCommandEvent&);
	void OnOpen(wxCommandEvent&);
	void OnQuit(wxCommandEvent&);
	
	void OnPreview(wxCommandEvent&);

	void OnWorldDefinitionCreated();
	void CreateWorldDefinition();
	void LoadWorldDefinition(const boost::filesystem::path& loadFrom);
	
	void DestroyWorldDefinition();

	void CreateSymbolViewPanel(const char* symbolName);
	void DestroySymbolViewPanel();

	void UpdateWindowCaption();
	
public:
	MainSystemFrame(const wxString& title, const wxPoint& pos, const wxSize& size, ResourceManager& resMan);
	virtual ~MainSystemFrame();

	DECLARE_EVENT_TABLE()
};

class AddSymbolDialog : public wxDialog
{
	wxTextCtrl* m_symNameBox;

public:
	AddSymbolDialog(wxWindow* parent, wxWindowID id);

	wxString GetSymbolName() const;
};

#endif
