#ifndef PRODUCTIONEDITDIALOG_H_
#define PRODUCTIONEDITDIALOG_H_

#include "QuadViewPanel.h"
#include "ProductionEditScene.h"
#include <wx/wx.h>

class ToolWorldDefinition;
class Production;

class ProductionEditDialog : public wxDialog
{
	ToolWorldDefinition& m_world;
	const ToolProduction* m_prod;

	QuadViewPanel* m_quadView;

	ProductionEditScene m_scene;

	wxComboBox* m_fadeTypeBox;

	wxListBox* m_componentsList;

	wxButton* m_addCompButton;
	wxButton* m_dupCompButton;
	wxButton* m_remCompButton;

	wxRadioButton* m_pmApproximations;
	wxRadioButton* m_pmWireframe;
	wxRadioButton* m_pmNone;

	wxCheckBox* m_drawBounds;

	wxTextCtrl* m_xPosText;
	wxTextCtrl* m_yPosText;
	wxTextCtrl* m_zPosText;
	wxButton* m_setPosButton;

	wxTextCtrl* m_scaleText;
	wxButton* m_setScaleButton;

	wxButton* m_rotateResetButton;

	enum
	{
		MY_COMPS_LIST = 300,
		ADD_COMP_BUTTON,
		DUP_COMP_BUTTON,
		REM_COMP_BUTTON,
		POS_SET_BUTTON,
		SCALE_SET_BUTTON,
		ROTATE_RESET_BUTTON,

		PM_APPROX,
		PM_WIREFRAME,
		PM_NONE,

		FM_COMBO,

		DRAW_BOUNDS,
	};

	void UpdateSelectedAttributes();

	void OnAddRSSym(wxCommandEvent&);
	void OnDupRSSym(wxCommandEvent&);
	void OnRemoveRSSym(wxCommandEvent&);

	void OnPosSet(wxCommandEvent&);
	void OnScaleSet(wxCommandEvent&);
	void OnRotateReset(wxCommandEvent&);

	void HandleFMChanged(wxCommandEvent&);
	
	void HandleApproxPreview(wxCommandEvent&);
	void HandleWireframePreview(wxCommandEvent&);
	void HandleNonePreview(wxCommandEvent&);
	
	void HandleDrawBoundsToggle(wxCommandEvent&);

	void OnCompSelectedInList(wxCommandEvent&);

	PotentialObject<Event<ToolWorldDefinition::ProductionRSAddedSig>::Connection> m_prodRSAddedConn;
	void HandleProductionRSAdded(const ToolProduction* prod, int index);

	PotentialObject<Event<ToolWorldDefinition::ProductionRSRemovedSig>::Connection> m_prodRSRemovedConn;
	void HandleProductionRSRemoved(const ToolProduction* prod, int index);

	PotentialObject<ProductionEditScene::SceneChangedEvent::Connection> m_sceneChangedConnection;
	void OnSceneChanged();

	void RecreateComponentsList();	

	PotentialObject<Event<void ()>::Connection> m_selectedChangedConn;
	void HandleSelectionChanged();

public:
	ProductionEditDialog(wxWindow* parent, const wxString& title, ToolWorldDefinition& world, const ToolProduction* prod);
	virtual ~ProductionEditDialog();

	DECLARE_EVENT_TABLE()
};

#endif
