#include "SymbolViewPanel.h"

#include "ToolWorldDefinition.h"
#include "QuadViewPanel.h"
#include "ProductionEditDialog.h"
#include "ComposerUtils.h"

#include <boost/bind.hpp>

#include <iostream>

BEGIN_EVENT_TABLE(SymbolViewPanel, wxPanel)
	EVT_BUTTON(APPROX_WITH_BUTTON, SymbolViewPanel::OnSelectApproxWith)
	EVT_BUTTON(CLEAR_APPROX_WITH_BUTTON, SymbolViewPanel::OnClearApproxWith)
	EVT_BUTTON(APPROX_THRESHOLD, SymbolViewPanel::OnSelectApproxThresh)
	EVT_BUTTON(ADD_PRODUCTION_BUTTON, SymbolViewPanel::OnAddProd)
	EVT_BUTTON(REM_PRODUCTION_BUTTON, SymbolViewPanel::OnRemoveProd)
	EVT_LISTBOX_DCLICK(MY_PRODS_LIST, SymbolViewPanel::OnMyProdsDClick)
	EVT_LISTBOX_DCLICK(PROD_APART_OF_LIST, SymbolViewPanel::OnProdsApartOfDClick)
END_EVENT_TABLE()

SymbolViewPanel::SymbolViewPanel(ToolWorldDefinition& world, const std::string& symName, wxWindow* parent, wxFrame* parentFrame)
: wxPanel(parent, -1)
, m_world(world)
, m_symName(symName)
, m_sym(m_world.GetSymbol(m_symName))
, m_parentFrame(parentFrame)
{
	wxBoxSizer* mainVertSizer = new wxBoxSizer(wxVERTICAL);
	SetSizer(mainVertSizer);

	{
		wxBoxSizer* horizSizer = new wxBoxSizer(wxHORIZONTAL);
		mainVertSizer->Add(horizSizer, 0, wxEXPAND|wxALL, 5);

		horizSizer->Add(new wxStaticText(this, -1, wxString("Approximate with:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

		m_approxWith = new wxButton(this, APPROX_WITH_BUTTON, wxString(""));
		horizSizer->Add(m_approxWith, 1, wxEXPAND|wxALL, 5);

		m_clearApproxWith = new wxButton(this, CLEAR_APPROX_WITH_BUTTON, wxString("Clear"));
		horizSizer->Add(m_clearApproxWith, 0, wxEXPAND|wxALL, 5);
	}
	{
		wxBoxSizer* horizSizer = new wxBoxSizer(wxHORIZONTAL);
		mainVertSizer->Add(horizSizer, 0, wxEXPAND|wxALL, 5);

		horizSizer->Add(new wxStaticText(this, -1, wxString("Approximation Threshold:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

		m_approxThresh = new wxButton(this, APPROX_THRESHOLD, wxString(""));
		horizSizer->Add(m_approxThresh, 1, wxEXPAND|wxALL, 5);
	}

	{
		wxBoxSizer* listSizers = new wxBoxSizer(wxHORIZONTAL);
		mainVertSizer->Add(listSizers, 1, wxEXPAND|wxALL, 5);

		{
			wxStaticBoxSizer* vertSizer = new wxStaticBoxSizer(wxVERTICAL, this, wxString("Productions"));
			listSizers->Add(vertSizer, 1, wxEXPAND|wxALL, 5);

			m_prodsList = new wxListBox(this, MY_PRODS_LIST);
			vertSizer->Add(m_prodsList, 1, wxEXPAND|wxALL, 5);

			vertSizer->Add(new wxStaticText(this, -1, wxString("Double-click a production to edit it")), 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

			wxBoxSizer* buttonHorizSizer = new wxBoxSizer(wxHORIZONTAL);
			vertSizer->Add(buttonHorizSizer, 0, wxALIGN_CENTER_HORIZONTAL, 0);
			
			m_addProdButton = new wxButton(this, ADD_PRODUCTION_BUTTON, wxString("Add"));
			buttonHorizSizer->Add(m_addProdButton, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

			m_remProdButton = new wxButton(this, REM_PRODUCTION_BUTTON, wxString("Remove"));
			buttonHorizSizer->Add(m_remProdButton, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

			// Refresh the list box with the production list for this symbol
			const Symbol::ProductionArray& prodArray = m_sym->GetSymbol()->GetLSProductionsArray();
			for(Symbol::ProductionArray::const_iterator iter = prodArray.begin(); iter != prodArray.end(); ++iter)
			{
				AddProductionToListbox((*iter), m_prodsList);
			}
		}

		{
			wxStaticBoxSizer* vertSizer = new wxStaticBoxSizer(wxVERTICAL, this, wxString("Productions Apart Of"));
			listSizers->Add(vertSizer, 1, wxEXPAND|wxALL, 5);

			m_prodsApartOfList = new wxListBox(this, PROD_APART_OF_LIST);
			vertSizer->Add(m_prodsApartOfList, 1, wxEXPAND|wxALL, 5);

			RefreshProdsApartOfList();
		}
	}

	RefreshSymbolProperties();

	new(m_prodAddedConn) Event<ToolWorldDefinition::ProductionAddedSig>::Connection(m_world.GetProductionAddedEventPort(), boost::bind<void>(&SymbolViewPanel::HandleProductionAdded, this, _1));
	new(m_prodRemovedConn) Event<ToolWorldDefinition::ProductionRemovedSig>::Connection(m_world.GetProductionRemovedEventPort(), boost::bind<void>(&SymbolViewPanel::HandleProductionRemoved, this, _1));
	new(m_prodRSAddedConn) Event<ToolWorldDefinition::ProductionRSAddedSig>::Connection(m_world.GetProductionRSAddedEventPort(), boost::bind(&SymbolViewPanel::HandleProductionRSAdded, this, _1));
	new(m_prodRSRemovedConn) Event<ToolWorldDefinition::ProductionRSRemovedSig>::Connection(m_world.GetProductionRSRemovedEventPort(), boost::bind(&SymbolViewPanel::HandleProductionRSRemoved, this, _1));
	new(m_symbolChangedConn) Event<ToolWorldDefinition::SymbolChangedSig>::Connection(m_world.GetSymbolChangedEventPort(), boost::bind<void>(&SymbolViewPanel::HandleSymbolChanged, this, _1));
}

SymbolViewPanel::~SymbolViewPanel()
{
}

void SymbolViewPanel::HandleProductionAdded(const ToolProduction* prod)
{
	AddProductionToListbox(prod->GetProduction(), m_prodsList);
}

void SymbolViewPanel::HandleProductionRemoved(const ToolProduction* prod)
{
	RemoveProductionFromListbox(prod->GetProduction(), m_prodsList);
	RemoveProductionFromListbox(prod->GetProduction(), m_prodsApartOfList);
}

void SymbolViewPanel::AddProductionToListbox(const Production* prod, wxListBox* listBox)
{
	// Formulate a string for this production
	listBox->Append(m_world.GetStringForProduction(prod), const_cast<Production*>(prod));
}

void SymbolViewPanel::RemoveProductionFromListbox(const Production* prod, wxListBox* listBox)
{
	for(int i=0; i<listBox->GetCount(); ++i)
	{
		const Production* listProd = (const Production*)listBox->GetClientData(i);
		if(listProd == prod)
		{
			listBox->Delete(i);
			break;
		}
	}
}


void SymbolViewPanel::OnSelectApproxWith(wxCommandEvent&)
{
	wxFileDialog fileDialogue(this, wxString("Choose a file"),  "", "",
								"OBJ Files (*.obj)|*.obj", wxFD_DEFAULT_STYLE);

	if(fileDialogue.ShowModal() == wxID_OK)
	{
		m_world.SetSymbolModel(m_sym, 
			ResourceIdentifier(
				boost::filesystem::path(fileDialogue.GetDirectory().c_str())
				/ boost::filesystem::path(fileDialogue.GetFilename().c_str())));
	}
}

void SymbolViewPanel::OnClearApproxWith(wxCommandEvent&)
{
	m_world.ClearSymbolModel(m_sym);
}

void SymbolViewPanel::OnAddProd(wxCommandEvent&)
{
	m_world.AddProduction(m_sym, Production::FM_NONE);
}

void SymbolViewPanel::OnRemoveProd(wxCommandEvent&)
{
	if(m_prodsList->GetSelection() == wxNOT_FOUND) return;

	const Production* prodSelected = (const Production*)m_prodsList->GetClientData(m_prodsList->GetSelection());
	ASSERT(prodSelected != NULL);
	const ToolProduction* toolProd = m_world.ToolProductionFromProduction(prodSelected);

	m_world.RemoveProduction(toolProd);
}

void SymbolViewPanel::OnMyProdsDClick(wxCommandEvent&)
{
	OnProductionListDoubleclicked(m_prodsList);
}

void SymbolViewPanel::OnProdsApartOfDClick(wxCommandEvent&)
{
	OnProductionListDoubleclicked(m_prodsApartOfList);
}



void SymbolViewPanel::OnProductionListDoubleclicked(wxListBox* list)
{
	int curSelection = list->GetSelection();
	if(curSelection == wxNOT_FOUND) return;

	const Production* prodSelected = (const Production*)list->GetClientData(curSelection);
	ASSERT(prodSelected != NULL);

	// Open up the production editor with this production
	ProductionEditDialog prodEditor(this, wxString("Production Editor"), m_world, m_world.ToolProductionFromProduction(prodSelected));
	if(prodEditor.ShowModal() == wxID_OK)
	{
	}
}


void SymbolViewPanel::HandleProductionRSAdded(const ToolProduction* prod)
{
	if(prod->GetProduction()->GetLS() == m_sym->GetSymbol())
	{
		RefreshProductionInListbox(prod->GetProduction(), m_prodsList);
	}

	if(m_sym->GetSymbol()->OnRSOf(prod->GetProduction()))
	{
		RefreshProductionInListbox(prod->GetProduction(), m_prodsApartOfList);
	}
}

void SymbolViewPanel::HandleProductionRSRemoved(const ToolProduction* prod)
{
	if(prod->GetProduction()->GetLS() == m_sym->GetSymbol())
	{
		RefreshProductionInListbox(prod->GetProduction(), m_prodsList);
	}


	if(!m_sym->GetSymbol()->OnRSOf(prod->GetProduction()))
	{
		RefreshProdsApartOfList();
	}
	else
	{
		RefreshProductionInListbox(prod->GetProduction(), m_prodsApartOfList);
	}
}

void SymbolViewPanel::RefreshProdsApartOfList()
{
	m_prodsApartOfList->Clear();

	const Symbol::ProductionArray& prodArray = m_sym->GetSymbol()->GetRSProductionsArray();
	for(Symbol::ProductionArray::const_iterator iter = prodArray.begin(); iter != prodArray.end(); ++iter)
	{
		AddProductionToListbox((*iter), m_prodsApartOfList);
	}
}

void SymbolViewPanel::RefreshProductionInListbox(const Production* prod, wxListBox* list)
{
	for(int i=0; i<list->GetCount(); ++i)
	{
		if((const Production*)list->GetClientData(i) == prod)
		{
			list->SetString(i, m_world.GetStringForProduction(prod));
			return;
		}
	}

	// If we're here, then we haven't found it yet, but it should be here, so add it
	AddProductionToListbox(prod, list);
}

void SymbolViewPanel::HandleSymbolChanged(const ToolSymbol* sym)
{
	if(sym == m_sym)
	{
		RefreshSymbolProperties();

	}
}

void SymbolViewPanel::RefreshSymbolProperties()
{
	{
		std::ostringstream oss;
		oss << m_sym->GetSymbol()->GetAreaApproximateThreshold();
		m_approxThresh->SetLabel(oss.str());
	}

	if(m_sym->IsModelSet())
	{
		m_approxWith->SetLabel(m_sym->GetModelResID().GetFilepath().leaf());
	}
	else
	{
		m_approxWith->SetLabel(wxString("[Nothing]"));
	}
}



class GetThresholdDialog : public wxDialog
{
	wxTextCtrl* m_text;
	wxButton* m_okButton;
	wxButton* m_cancelButton;
public:
	GetThresholdDialog(wxWindow* parent, const wxString& initialThreshold)
		: wxDialog(parent, -1, wxString("Enter Threshold:"), wxDefaultPosition, wxSize(200, 120))
	{
		wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
		SetSizer(mainSizer);

		wxBoxSizer* topSizer = new wxBoxSizer(wxHORIZONTAL);
		mainSizer->Add(topSizer, 1, wxEXPAND|wxALL, 5);

		topSizer->Add(new wxStaticText(this, -1, wxString("Threshold:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

		m_text = new wxTextCtrl(this, -1, initialThreshold);
		topSizer->Add(m_text, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

		wxBoxSizer* bottomSizer = new wxBoxSizer(wxHORIZONTAL);
		mainSizer->Add(bottomSizer, 1, wxEXPAND|wxALL, 5);


		bottomSizer->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), 1, wxEXPAND|wxALL, 5);
	};

	wxString GetThresholdString() const
	{
		return m_text->GetValue();
	}

	void OnOkay(wxCommandEvent& event)
	{
		if(ValidateFloatString(m_text->GetValue()))
		{
			event.Skip();
		}
		else
		{
			::wxMessageBox(wxString("Input must be a string of decimal digits with at most one period.  Ie: 1, 3., .5, 03.5"), wxString("Invalid Input"), wxOK|wxICON_ERROR);
		}
	}

	DECLARE_EVENT_TABLE()
};
BEGIN_EVENT_TABLE(GetThresholdDialog, wxDialog)
	EVT_BUTTON(wxID_OK, GetThresholdDialog::OnOkay)
END_EVENT_TABLE()

void SymbolViewPanel::OnSelectApproxThresh(wxCommandEvent&)
{
	GetThresholdDialog threshDialogue(this, m_approxThresh->GetLabel());
	if(threshDialogue.ShowModal() == wxID_OK)
	{
		float num;
		std::istringstream iss(threshDialogue.GetThresholdString().c_str());
		iss >> num;

		m_world.SetSymbolApproxThreshold(m_sym, num);
	}	
}
