#include "wxWidgetGraphicsContext.h"

#include <GLee.h>

#include <string.h>

#include "R3Utils.h"

#include <wx/glcanvas.h>


wxWidgetGraphicsContext::wxWidgetGraphicsContext(wxGLCanvas& canvas)
: m_canvas(canvas)
, m_near(0.01f)
, m_far(500.0f)
{
	int width;
	int height;
	canvas.GetClientSize(&width, &height);
	m_width = width;
	m_height = height;

	m_wxContext = new wxGLContext(&m_canvas);
#ifdef PLAT_WIN32
	// this is the stupidest.  I don't even know why this is needed, but without it,
	// deleting a rendering context makes subsequent created rendering contexts act
	// stupid.  IE: can't resize, viewport is fucked.  Don't even ask me why, or
	// why this fixes it.
	m_wxContext->SetCurrent(m_canvas);
	wglMakeCurrent(NULL, NULL);
#endif

	{
		Mutex::ScopedLock lock(m_initMutex);

		m_commandQueue = new CommandQueue(100, 20000, false);
		m_thread = new Thread(boost::bind<void>(&wxWidgetGraphicsContext::Run, this));

		m_initCond.Wait(m_initMutex);
	}
}

wxWidgetGraphicsContext::~wxWidgetGraphicsContext()
{
	delete m_commandQueue;
	delete m_thread;

	delete m_wxContext;
}

void wxWidgetGraphicsContext::Run()
{
	{
		Mutex::ScopedLock lock(m_initMutex);

		m_wxContext->SetCurrent(m_canvas);

		m_initCond.SignalAll();
	}

	m_commandQueue->ProcessCommands();
}

void wxWidgetGraphicsContext::ProcessCommand(RenderFrameCommand& cmd)
{
	OpenGLRenderContext::RenderOptions glOptions;

	m_glContext.RenderGraphicsFrame(cmd.frame, cmd.proj, glOptions);
	m_canvas.SwapBuffers();
}

void wxWidgetGraphicsContext::RenderGraphicsFrame(GraphicsFrame& frame, const GraphicsContext::RenderOptions& options)
{
	ADDCOMMAND(*m_commandQueue, RenderFrameCommand(*this, frame, options, options.GetProjectionMatrix(), frame.MakeFreeze()));
}

void wxWidgetGraphicsContext::ProcessCommand(ResizeViewportCommand& cmd)
{
	m_glContext.SetViewportSize(cmd.width, cmd.height);
}

void wxWidgetGraphicsContext::ResizeScreen(int width, int height)
{
	m_width = width;
	m_height = height;

	ADDCOMMAND(*m_commandQueue, ResizeViewportCommand(*this, width, height));
}

void wxWidgetGraphicsContext::FlushWait()
{
}

int wxWidgetGraphicsContext::GetWidth()
{
	return m_width;
}

int wxWidgetGraphicsContext::GetHeight()
{
	return m_height;
}

