#ifndef WXWIDGETKEYBOARDINPUT_H_
#define WXWIDGETKEYBOARDINPUT_H_

#include <wx/wx.h>
#include <KeyboardInput.h>

#include <PotentialObject.h>

#include <set>

/** Converts wxWidget keyboard key press events in to our own format and returns them
 *	through the KeyboardInput interface.  In this way, any system that accepts the
 *	KeyboardInput interface will now also be able to handle wxWidget events.
 */
class wxWidgetKeyboardInput : public KeyboardInput
{
	class EventHandler : public wxEvtHandler
	{
		wxWidgetKeyboardInput& m_parent;

		void OnKeyDown(wxKeyEvent& event);
		void OnKeyUp(wxKeyEvent& event);
	public:
		EventHandler(wxWidgetKeyboardInput& parent)
			: m_parent(parent)
		{};

		DECLARE_EVENT_TABLE()
	};
	PotentialObject<EventHandler> m_handler;

	void OnKeyDown(wxKeyEvent& event);
	void OnKeyUp(wxKeyEvent& event);

	std::set<int> m_keysDown;

public:
	/**
	 *	@param handler The wxWidgetKeyboardInput object will link in with the passed in wxWidget
	 *					event handler's event handling list.  We will then listen for keyboard events.
	 */
	wxWidgetKeyboardInput(wxEvtHandler& handler);
};

#endif
